import re
from typing import List

from nltk.tokenize import regexp_tokenize


def tok(sentence: str) -> List[str]:
    regex_str = r"[\[\(\{][a-zA-Z]* *\/*,*-* *[0-9][0-9]? *\/*,*-* *[a-zA-Z]*[\]\)\}]|\w+"
    return regexp_tokenize(sentence, regex_str)


def get_feature_1(sentence: str, index: int) -> str:
    """Token to the left."""
    tokens = tok(sentence)
    return tokens[index - 1].lower() if index >= 1 else ""


def get_feature_2(sentence: str, index: int) -> str:
    """Token to the right."""
    tokens = tok(sentence)
    return tokens[index + 1].lower() if index < len(tokens) - 1 else ""


def get_feature_3(token1: str, token2: str) -> str:
    """Conjunction of features 1 and 2."""
    if token1 == "":
        token1 = "..."
    if token2 == "":
        token2 = "..."
    return token1 + "^" + token2


def get_feature_4(sentence: str, token: str) -> int:
    """If gender token is the first token in title."""
    tokens = tok(sentence)
    return 1 if tokens[0] == token else 0


def get_feature_5(token: str) -> str:
    """Returns the alphabetic part of a token"""
    alphabetic = re.sub(r"[^A-Za-z]+", "", token.lower())
    return alphabetic if alphabetic != "" else "..."



import re
from typing import Optional

import joblib
import numpy as np
import pkg_resources

from .features import (
    get_feature_1,
    get_feature_2,
    get_feature_3,
    get_feature_4,
    get_feature_5,
    tok,
)

REGEX_STR = (
    r"[\[\(\{] *[a-zA-Z]* *\/*,*-* *[0-9][0-9]? *\/*,*-* *[a-zA-Z]* *[\]\)\}]"
)


def generate_features(title: str, index: str):
    """
    Given a string, generate features necessary for prediction.
    """
    token = tok(title)[index]
    feature_1 = get_feature_1(title, index)
    feature_2 = get_feature_2(title, index)
    feature_3 = get_feature_3(feature_1, feature_2)
    feature_4 = get_feature_4(title, token)
    feature_5 = get_feature_5(token)
    features = {
        "feature_1": feature_1,
        "feature_2": feature_2,
        "feature_3": feature_3,
        "feature_4": feature_4,
        "feature_5": feature_5,
    }
    return features


class Iama:
    def __init__(self):
        self._model = None
        self._dv = None
        self.load()

    def predict(self, title: str) -> Optional[str]:
        """
        Given a title, predict the token (e.g. [27M]) that refers
        to the post's author.
        """
        X = []
        tokens = []
        tok_title = tok(title)
        has_token = False
        for idx in range(len(tok_title)):
            if re.match(REGEX_STR, tok_title[idx]):
                has_token = True
                tokens.append(tok_title[idx])
                X.append(generate_features(title, idx))

        if not has_token:
            return None

        X_encoded = self._dv.transform(X)
        probs_one = self._model.predict_log_proba(X_encoded)[:, 1]
        idx_max = np.argmax(probs_one)
        return tokens[idx_max]

    def load(self):
        self._model = joblib.load(
            pkg_resources.resource_filename(__name__, "models/iama.pkl")
        )
        self._dv = joblib.load(
            pkg_resources.resource_filename(
                __name__, "models/dict_vectorizer.pkl"
            ),
        )
        return self



import os

import setuptools

_THIS_DIR = os.path.dirname(os.path.realpath(__file__))

with open(os.path.join(_THIS_DIR, "README.md"), encoding="utf-8") as f:
    _LONG_DESCRIPTION = f.read().strip()


def main():
    setuptools.setup(
        name="iama",
        version="0.1.4",
        author="Daniel You",
        author_email="daniel.you@jerichoapps.org",
        description="Classifying r/relationships post titles by gender.",
        long_description=_LONG_DESCRIPTION,
        long_description_content_type="text/markdown",
        url="https://github.com/youdaniel/iama",
        packages=setuptools.find_packages(),
        license="Apache 2.0",
        python_requires=">=3.6",
        classifiers=[
            "Development Status :: 3 - Alpha",
            "License :: OSI Approved :: Apache Software License",
            "Operating System :: OS Independent",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3 :: Only",
        ],
        install_requires=["nltk", "setuptools", "scikit-learn", "numpy"],
    )


if __name__ == "__main__":
    main()
